@@ -167,6 +167,7 @@ class Recognizer(WebSocketClient):
         self.__speech_synthesis_voice_name = speech_config.speech_synthesis_voice_name
         self.__output_format = speech_config.output_format
         self.__speech_synthesis_output_format_id = speech_config.speech_synthesis_output_format_id
+        self.__recognizer_semaphore = threading.Semaphore(0)
 
         # load audio_config
         self.__input_stream = None
@@ -217,7 +218,7 @@ class Recognizer(WebSocketClient):
             self.send("EOS")
 
     def start_continuous_recognition_async(self):
-        """Start speech recognition"""
+        """Start continuous speech recognition in asynchronized way"""
         self.is_running = True
         self.connect()
         recognition_thread = threading.Thread(
@@ -226,6 +227,11 @@ class Recognizer(WebSocketClient):
         recognition_thread.daemon = True
         recognition_thread.start()
 
+    def start_continuous_recognition(self):
+        """Start continuous speech recognition in synchronized way"""
+        self.start_continuous_recognition_async()
+        self.__recognizer_semaphore.acquire()
+
     def thread_start_continuous_recognition_async(self):
         """
         Start a thread to start continuous recognition
@@ -346,7 +352,11 @@ class Recognizer(WebSocketClient):
     def stop_continuous_recognition_async(self):
         """Stop recognition"""
         self.is_running = False
-        self.ngchat_stt_loop.close()
+
+    def stop_continuous_recognition(self):
+        """Stop recognition"""
+        self.is_running = False
+        self.__recognizer_semaphore.release()
 
     @property
     def speech_config(self) -> SpeechConfig:
@@ -616,7 +626,7 @@ class ResultFuture():
 
 
 class SlothWebsocketParam(object):
-    def __init__(self, app_id, api_key, api_secret, text):
+    def __init__(self, app_id, api_key, api_secret, text, is_ssml):
         self.app_id = app_id
         self.api_key = api_key
         self.api_secret = api_secret
@@ -625,7 +635,8 @@ class SlothWebsocketParam(object):
         self.common_args = {"app_id": self.app_id}
         self.business_args = {"aue": "raw", "auf": "audio/L16;rate=16000", "vcn": "xiaoyan", "tte": "utf8"}
         self.data = {"status": SLOTH_WEBSOCKET_STATUS_LAST_FRAME,
-                     "text": str(base64.b64encode(self.text.encode('utf-8')), "UTF8")}
+                     "text": str(base64.b64encode(self.text.encode('utf-8')), "UTF8"),
+                     "ssml": is_ssml}
 
     def create_url(self, host_url):
         now = datetime.now()
@@ -684,12 +695,13 @@ class SlothWebsocketClient():
         audio.writeframes(audio_pcm)
         audio.close()
 
-    def speak_text(
+    def sloth_speak_text(
             self,
-            input_text: str):
+            input_text: str,
+            is_ssml='False'):
         self.ws_param = SlothWebsocketParam(app_id='appid_001', api_key='appkey_001',
                                             api_secret='api_secret',
-                                            text=input_text)
+                                            text=input_text, is_ssml=is_ssml)
         ws_url = self.ws_param.create_url(host_url=self.__host)
         self.ws.url=ws_url
         self.ws.run_forever(sslopt={"cert_reqs": ssl.CERT_NONE})
@@ -706,11 +718,12 @@ class SlothWebsocketClient():
         self.semaphore.release()
         return self.speech_synthesis_result
 
-    def speak_text_async(self,
-                         input_text: str):
+    def sloth_speak_text_async(self,
+                         input_text: str,
+                         is_ssml='False'):
         task_speak = threading.Thread(
-            target = self.speak_text,
-            args = (input_text,)
+            target=self.sloth_speak_text,
+            args=(input_text, is_ssml)
         )
         task_speak.daemon = True
         task_speak.start()
@@ -731,7 +744,7 @@ class SlothWebsocketClient():
 
                 if self.synthesis_completed is not None:
                     if self.synthesis_completed.is_set is True:
-                        self.synthesis_completed.callback()
+                        self.synthesis_completed.callback(audio)
 
                 svc_logger.info("Sloth websocket: ws is closed")
             if code != 0:
@@ -743,7 +756,7 @@ class SlothWebsocketClient():
 
                     if self.synthesizing is not None:
                         if self.synthesizing.is_set is True:
-                            self.synthesizing.callback()
+                            self.synthesizing.callback(audio)
 
         except Exception as e:
             svc_logger.error(f"Sloth websocket exception: {e}")
@@ -830,7 +843,7 @@ class SpeechSynthesizer(SlothWebsocketClient):
 
         :return: A SpeechSynthesisResult.
         """
-        return super().speak_text(text)
+        return super().sloth_speak_text(text)
 
     def speak_ssml(self, ssml: str) -> SpeechSynthesisResult:
         """
@@ -838,7 +851,7 @@ class SpeechSynthesizer(SlothWebsocketClient):
 
         :return: A SpeechSynthesisResult.
         """
-        raise NotImplementedError
+        return super().sloth_speak_text(ssml, 'True')
 
     def speak_text_async(self, text: str) -> ResultFuture:
         """
@@ -846,7 +859,7 @@ class SpeechSynthesizer(SlothWebsocketClient):
 
         :return: A future with SpeechSynthesisResult.
         """
-        return super().speak_text_async(text)
+        return super().sloth_speak_text_async(text)
 
     def speak_ssml_async(self, ssml: str) -> ResultFuture:
         """
@@ -854,7 +867,7 @@ class SpeechSynthesizer(SlothWebsocketClient):
 
         :return: A future with SpeechSynthesisResult.
         """
-        raise NotImplementedError
+        return super().sloth_speak_text_async(ssml, 'True')
 
     def start_speaking_text(self, text: str) -> SpeechSynthesisResult:
         """
